{% extends "base.html" %}
{% block content %}

<div data-role="page" data-theme="b" id="khan-video">
    <div data-role="header">
        <a href="/iiab/" data-icon="home" data-iconpos="notext" data-direction="reverse">{% trans %}Home{% endtrans %}</a>
        <h1>
            {% for level in breadcrumbs %}
            {% if not loop.first %}&gt;{% endif %} {{ level[1] }}
            {% endfor %}
        </h1>
    </div>
    <div data-role="content">

        <video width="100%" id="video" autobuffer controls>
        <source src="{{h264}}">
        <source src="{{webm}}" type="video/webm">
        </video>

        <!-- script>
        var video = document.getElementById('video');
        video.addEventListener('click',function(){
        video.play();
        },false);
        </script -->
    </div>
</div>

{% endblock %}
