{% extends "base.html" %}
{% block content %}

<div data-role="page" data-theme="b" id="khan-index">
    <div data-role="header">
        <a href="/iiab/" data-icon="home" data-iconpos="notext" data-direction="reverse">{% trans %}Home{% endtrans %}</a>
        <h1>{% trans %}Videos from Khan Academy{% endtrans %}</h1>
    </div>
    <div data-role="content">
        <ul data-role="listview" data-divider-theme="b" data-inset="true">
            {% for child in children %}
            <li data-theme="c">
            <a href="{{ child['url'] }}" data-transition="slide">
                {{ child['index'] }} - {{ child['name'] }}
            </a>
            </li>
            {% endfor %}
        </ul>
    </div>
</div>

{% endblock %}
