.leaflet-center {
	width: 40%;
	margin-left: auto;
	margin-right: auto;
	position: relative;
}
.leaflet-control-geosearch, .leaflet-control-geosearch ul {
	border-radius: 7px;
	background: none repeat scroll 0 0 rgba(0, 0, 0, 0.25);
	margin: 10px 0 0 0;
	padding: 5px;
	width: 100%;
	height: auto;
}
.leaflet-control-geosearch-msg ul {
	list-style: none;
	display: none;
	height: auto;
	background: none;
	padding: 0;
}
.leaflet-control-geosearch ul li {
	background: none repeat scroll 0 0 rgba(255, 255, 255, 0.75);
    border-radius: 4px;
    margin: 2px 0;
    padding: 4px;
    font: 12px arial;
    text-indent: 4px;
}
.leaflet-container .leaflet-control-geosearch input {
	width: 100%;
	height: 28px;
	padding: 0;
	text-indent: 8px;
	background: rgba(255, 255, 255, 0.75);			
	border-radius: 4px;
	border: none;
}
.leaflet-geosearch-autocomplete {
    position:absolute;
    top:100%;
    left:0;
    float:left;
    width:100%;
    box-shadow: 0 0 8px rgba(0,0,0,0.4);
    -webkit-border-radius: 5px;
    -webkit-border-top-left-radius: 0;
    -moz-border-radius: 5px;
    -moz-border-radius-topleft: 0;
    border-radius: 5px;
    border-top-left-radius: 0;
    background-color: rgba(0, 0, 0, 0.25);
    z-index:1010;
    overflow-y:auto;
    overflow-x:hidden;
}
.leaflet-geosearch-suggestion {
    font-size:.85em;
    margin:2px;
    padding:2px;
    display:block;
    color:black;
    background: #ddd;
    border-radius:.25em;
    text-decoration:none;
    white-space:nowrap;
    font-size:.85em;
    vertical-align:center;
}
.leaflet-geosearch-suggestion:hover, .leaflet-geosearch-suggestion-selected {
    background-color: #fff;
}
/*icon css adapted from leaflet-search*/
.leaflet-control-geosearch-button-submit-container {
    background: none repeat scroll 0 0 #AAAAAA;
    border-radius: 4px 4px 4px 4px;
    height: 24px;
    margin: 2px 0;
    position: absolute;
    right: 8px;
    width: 27px;
}
.leaflet-control-geosearch-button-submit-container .leaflet-geosearch-submit-button {
    display: block;
    margin: 2px 0;
    position: absolute;
    text-decoration: none;
    background: url("../images/search-icon.png") no-repeat scroll 0 0 transparent;
    border-radius: 4px 4px 4px 4px;
    height: 22px;
    width: 22px;
    right: 2px;
}
.leaflet-control-geosearch-button-submit-container .leaflet-geosearch-submit-button:hover {
    background: url('../images/search-icon.png') no-repeat 0 -21px;
}
.leaflet-geosearch-cancel-button {
    background: url("../images/search-icon.png") no-repeat scroll 0 -42px transparent;
    height: 18px;
    width: 18px;
    margin-top: 5px;
    filter: alpha(opacity=80);
    opacity: 0.8;
    right: 42px;
    display: none; /*initially hidden*/
    position: absolute;
    text-decoration: none;
}
.leaflet-geosearch-cancel-button:hover {
    filter: alpha(opacity=100);
    opacity: 1;
}
