{% extends "base.html" %}

  
{% block head %}
<script>
    languages_url = "{{ url_for('settings.languages_json') }}";
    user_language_url = "{{ url_for('settings.user_language') }}";
</script>
<script src="{{ static('js/settings.js') }}"></script>
{% endblock %}

{% block content %}

<!-- Settings -->
<div data-role="page">
    <div data-theme="a" data-role="header">
        <a data-role="button" data-direction="reverse" data-rel="back" data-transition="slide"
        href="#page1" class="ui-btn-left">
            {% trans %}Back{% endtrans %}
        </a>
        <h3>
            {% trans %}Settings{% endtrans %}
        </h3>
    </div>
    <div data-role="content">
        <div data-role="fieldcontain">
            <label for="selectmenu3">
                {% trans %}Language{% endtrans %}
            </label>
            <select id="language_select" name="language_select"></select>
        </div>
        <!--
        <div data-role="fieldcontain">
            <fieldset data-role="controlgroup">
                <label for="toggleswitch1">
                    {% trans %}Chat{% endtrans %}
                </label>
                <select name="toggleswitch1" id="toggleswitch1" data-theme="" data-role="slider">
                    <option value="off">
                        {% trans %}Off{% endtrans %}
                    </option>
                    <option value="on">
                        {% trans %}On{% endtrans %}
                    </option>
                </select>
            </fieldset>
        </div>
        -->
        <h2>
            {% trans %}About{% endtrans %}
        </h2>
        <h4>
            {% trans %}Software Version{% endtrans %}: {{ software_version }}
        </h4>
        <h4>
            {% trans %}Data Version{% endtrans %}: 20130214
        </h4>
        <h2>
            {% trans %}Licenses{% endtrans %}
        </h2>
        <div>
            <p>
                <b>
                    {% trans %}<a href="http://www.wikipedia.org/">Wikipedia</a>&reg; is a registered trademark of the <a href="http://www.wikimediafoundation.org/">Wikimedia Foundation, Inc.</a>, a non-profit organization. Text is available under the <a href="http://en.wikipedia.org/wiki/Wikipedia:Text_of_Creative_Commons_Attribution-ShareAlike_3.0_Unported_License">Creative Commons Attribution-ShareAlike License ("CC BY-SA")</a>.{% endtrans %}
                </b>
                <br>
            </p>
        </div>
        <div>
            <p>
                <b>
                    {% trans %}Maps - &copy; OpenStreetMap contributors - <a href="http://www.openstreetmap.org/">OpenStreetMap</a> is open data, licensed under the <a href="http://opendatacommons.org/licenses/odbl/">Open Data Commons Open Database License (ODbL).</a> <a href="http://www.openstreetmap.org/copyright">Copyright and License</a>.{% endtrans %}
                </b>
                <br>
            </p>
        </div>
        <div>
            <p>
                <b>
                    {% trans %}Map cartography in OpenStreetMap map tiles is licensed under the <a href="http://creativecommons.org/licenses/by-sa/2.0/">Creative Commons Attribution-ShareAlike 2.0 license (CC BY-SA)</a>{% endtrans %}.
                </b>
                <br>
            </p>
        </div>
        <div>
            <p>
                <b>
                    {% trans %}The KHAN ACADEMY&reg; word mark is a trademark of Khan Academy, Inc.  NOTE: All Khan Academy content is available for free at <a href="http://www.khanacademy.org/">www.khanacademy.org</a>. Videos are licensed under the <a href="http://creativecommons.org/licenses/by-nc-sa/3.0/us/">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 (CC BY-NC-SA) license</a>.{% endtrans %}
                </b>
                <br>
            </p>
        </div>
        <div>
            <p>
                <b>
                    {% trans %}<a href="http://internet-in-a-box.org/">Internet-in-a-Box</a> software - &copy; 2013 <a href="http://www.humaneinfo.com/">Humane Informatics, LLC</a>. All rights reserved. This software is licensed under the <a href="http://opensource.org/licenses/BSD-2-Clause">BSD 2-clause license</a>.{% endtrans %}
                    <br>
                </b>
            </p>
        </div>
    </div>
</div>

{% endblock %}
