<!--[if lt IE 7]>
    <p class="chromeframe">{% trans %}You are using an <strong>outdated</strong> browser. Please <a href="http://browsehappy.com/">upgrade your browser</a> or <a href="http://www.google.com/chromeframe/?redirect=true">activate Google Chrome Frame</a> to improve your experience.{% endtrans %}</p>
<![endif]-->
