{% extends "zim/base.html" %}

{% block head %}
  <style type='text/css'>
      <!--
      /* CSS for Wikipedia iframe in zim_iframe.html */
      .iframe-wrap {
          position: fixed;
          top: 45px;
          left: 0;
          width: 100%;
          height: 100%;
      }
      .wiki-iframe {
          height: 90%;
          width: 100%;
      }
      @media only screen and (min-width: 900px) {
          .iframe-wrap {
              width: 900px !important;
              left: 50%;
              margin-left: -450px;
          }
      }
      -->
   </style>
{% endblock %}


{% block zim_content %}
    <div class="iframe-wrap">
        <iframe class="wiki-iframe" frameborder="0" height="100%" width="100%" src="{{ url }}">
        </iframe>
    </div>
{% endblock %}
