/* Author: Wilson Xu */

function hide_flask_message_container() {
    $('#flash_message_container').slideUp('fast');
}

$(document).ready(function() {
    /* Show and hide flash message. */
    //$('#flash_message_container').slideDown(function() {
        //setTimeout(hide_flask_message_container, 3000);
    //});
})
